/* Copyright (C) 1989 Aladdin Enterprises.  All rights reserved.
   Distributed by Free Software Foundation, Inc.

This file is part of Ghostscript.

Ghostscript is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the Ghostscript General Public License for full details.

Everyone is granted permission to copy, modify and redistribute
Ghostscript, but only under the conditions described in the Ghostscript
General Public License.  A copy of this license is supposed to have been
given to you along with Ghostscript so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.  */

/* gxdevice.h */
/* Device description structure for GhostScript library */

/* The structure is divided into three parts: procedures that are
 * shared by all instances of each device, parameters that are
 * present in all devices but may be different for each instance,
 * and device-specific parameters.  The gx_device_common macro
 * defines the common structure elements, with the intent that
 * devices define and export a structure along the following lines:
 *
 *	typedef struct my_device_s {
 *		gx_device_common;
 *		... device-specific parameters ...
 *	} my_device;
 *	my_device foo_device = {
 *		sizeof(my_device),		* params_size
 *		{ ... procedures ... },		* procs
 *		... generic parameter values ...
 *		... device-specific parameter values ...
 *	};
 *
 * All the device procedures are called with the device as the first
 * argument.  Since each device type is actually a different structure
 * type, the device procedures must be declared as taking a gx_device *
 * as their first argument, and must cast it to my_device * internally.
 */

typedef struct gx_device_s gx_device;
typedef struct gx_device_procs_s gx_device_procs;

/* Structure for device procedures */
struct gx_device_procs_s {

	/* All the procedures except map_rgb_color return */
	/* 0 on success, or -1 on invalid arguments. */

	/* Some procedures are noted as optional below. */
	/* If a device doesn't supply an optional procedure proc, */
	/* the entry in the procedure structure should be */
	/* gx_default_proc, e.g. gx_default_tile_rectangle. */
	/* The device procedure can also call this procedure */
	/* if it doesn't implement the function for particular */
	/* values of the arguments. *

	/* Open the device.  OPTIONAL */
	int (*open_device)(P1(gx_device *));

	/* Close the device.  OPTIONAL */
	int (*close_device)(P1(gx_device *));

	/* Map a r-g-b color to a device color code.  OPTIONAL. */
	/* The default returns the brightness, i.e. max(r, g, b). */
	int (*map_rgb_color)(P4(gx_device *, unsigned short, unsigned short, unsigned short));
		/* r, g, b -> color */

	/* Map a device color code to r-g-b values.  OPTIONAL. */
	/* The default sets r, g, b = color. */
	int (*map_color_rgb)(P3(gx_device *, int, unsigned short *));
		/* color -> [r, g, b] */

	/* Synchronize the display -- any pending calls take effect. */
	/* OPTIONAL */
	int (*sync_output)(P1(gx_device *));

	/* Fill a rectangle */
	int (*fill_rectangle)(P6(gx_device *, int, int, int, int, int));
		/* x, y, width, height, color */

	/* Tile a rectangle.  OPTIONAL */
	int (*tile_rectangle)(P8(gx_device *, gx_bitmap *, int, int, int, int, int, int));
		/* tile, x, y, width, height, 0-color, 1-color */

	/* Copy a monochrome bitmap (stored left-to-right, top-to-bottom). */
	/* If a color is -1, it means don't change the bit. */
	int (*copy_mono)(P10(gx_device *, unsigned char *, int, int, int, int, int, int, int, int));
		/* data, data x, raster, x, y, width, height, 0-color, 1-color */

	/* Copy a color bitmap, with multiple bits/pixel.  If the color */
	/* value equals color_transparent, it means don't change the bit. */
	int (*copy_color)(P9(gx_device *, unsigned char *, int, int, int, int, int, int, int));
		/* data, data x, raster, x, y, width, height, transparent-color */

	/* Draw a minimum-thickness line.  OPTIONAL */
	int (*draw_line)(P6(gx_device *, int, int, int, int, int));
		/* x0, y0, x1, y1, color */

	/* Fill a trapezoid.  OPTIONAL */
	int (*fill_trapezoid)(P8(gx_device *,
				 int, int, int, int, int, int, int));
		/* x0, y0, width0, x1, y1, width1, color */

	/* Tile a trapezoid.  OPTIONAL */
	int (*tile_trapezoid)(P10(gx_device *, gx_bitmap *,
				  int, int, int, int, int, int, int, int));
		/* tile, x0 ... width1, 0-color, 1-color */

};

/* Default implementations of optional procedures */
extern int
  gx_default_open_device
    (P1(gx_device *)),
  gx_default_close_device
    (P1(gx_device *)),
  gx_default_map_rgb_color
    (P4(gx_device *, unsigned short, unsigned short, unsigned short)),
  gx_default_map_color_rgb
    (P3(gx_device *, int, unsigned short *)),
  gx_default_sync_output
    (P1(gx_device *)),
  gx_default_tile_rectangle
    (P8(gx_device *, gx_bitmap *, int, int, int, int, int, int)),
  gx_default_draw_line
    (P6(gx_device *, int, int, int, int, int)),
  gx_default_fill_trapezoid
    (P8(gx_device *, int, int, int, int, int, int, int)),
  gx_default_tile_trapezoid
    (P10(gx_device *, gx_bitmap *, int, int, int, int, int, int, int, int));

/* Structure for generic device description */
#define gx_device_common\
	int params_size;		/* size of this structure */\
	gx_device_procs *procs;\
	int width;			/* width in pixels */\
	int height;			/* height in pixels */\
	int has_color;			/* true if device supports color */\
	unsigned short max_rgb_value;	/* max r, g, b value */\
	int bits_per_color_pixel;	/* for copy_color */\
	int bits_from_MSB		/* true if bit-big-endian */\
/* A generic device */
struct gx_device_s {
	gx_device_common;
};
/* A 'memory' device -- a stored bitmap */
typedef struct gx_device_memory_s gx_device_memory;
struct gx_device_memory_s {
	gx_device_common;
	int raster;			/* bytes per scan line */
	unsigned char *base;
};
extern gx_device_memory mem_device;
